//
//  RobotSampleViewController.h
//  RobotSample
//
//	Feel free to modify the class and let me know what you do.
//
//  Created by Paul Solt, paulsolt@gmail.com on 8/2/09.
//  Copyright RIT 2009. All rights reserved.
//

#import <UIKit/UIKit.h>
#import "Robot.h"

//	The RobotSampleViewController is a class that bridges the gap between Player/Stage
//		and iPhone. It is a basic class that can be extended and has a bunch of useful features
//		to extend.
//
//	1. The RobotSampleViewController creates a new thread that runs the Robot class's 
//		@selector(robotRun:) method and passes an instance of itself to the robot.
//
//	2. The RobotSampleViewController implements two callback methods required by
//		the @selector(robotRun:)
//			- (void) displayText:(NSString *)text; 
//			- (void) connectionClosedWithMessage:(NSString *)message;
//		These methods provide a way for the Player/Stage robot code to communicate with the
//		user interface on iPhone. Similar methods can be created to pass data, but the
//		data must be packaged in an Objective-C object.
//
//	3. The RobotSampleViewController reads the server address from the Settings.bundle,
//		which is created by the Settings App on the iPhone. You can open the Settings for the
//		application when it's installed on a device/simulator and change this default address
//		without recompiling the applicaiton. (Clean/uninstall on any settings changes)
//
//	4. The RobotSampleViewController creates a NSTimer object and registers an update:
//		method. This method will provide a way to do logic at a fixed time interval on the 
//		user interface.
//
@interface RobotSampleViewController : UIViewController {
	
	// Basic TextViews that are attached to the .xib files
	IBOutlet UITextView *serverTextView;
	IBOutlet UITextView *statusTextView;
	
	// The server address to connect to
	NSString *server;
	
	// A robot object that wraps the Player/Stage code in Objective-C
	Robot *robot;
	
	// A timer object to trigger timer events on a regular interval
	NSTimer *timer;	
}

@property (nonatomic, retain) NSString *server;

//	The update method that is called every 60th of a second. Put any update
//		logic within this method.
- (void)update:(NSTimer *) theTimer;

//	Displays text in the statusTextView. This method is called any time there is an
//		update to the display from the Robot robotRun: method.
- (void)displayText:(NSString *)text;

//	Displays text in the statusTextView on an error or connection close. Add additional
//		exit logic to cleanup the iPhone code.
- (void)connectionClosedWithMessage:(NSString *)message;

//	Programatically load the default settings from the Default settings. This is a workaround
//		solution for settings loading on the iPhone apps first install on a device/simulator.
//	*Note*: Default settings are not loaded on the programs first run, but remain nil until either 
//		this method is called or the Settings App is opened.
- (void)registerDefaultsFromSettingsBundle;

@end

