//
//  RobotSampleViewController.m
//  RobotSample
//
//	Feel free to modify the class and let me know what you do.
//
//  Created by Paul Solt, paulsolt@gmail.com on 8/2/09.
//  Copyright RIT 2009. All rights reserved.
//

#import "RobotSampleViewController.h"

@implementation RobotSampleViewController

@synthesize server;

- (void)update:(NSTimer*)theTimer {
}

- (void)displayText:(NSString *)text {
	NSLog(text);
	statusTextView.text = text;
}

- (void)connectionClosedWithMessage:(NSString *)message {
	NSLog(@"The robot connection was closed: %@\n", message);
	statusTextView.text = message;
}

/*
// The designated initializer. Override to perform setup that is required before the view is loaded.
- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil {
    if (self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil]) {
        // Custom initialization
    }
    return self;
}
*/

/*
// Implement loadView to create a view hierarchy programmatically, without using a nib.
- (void)loadView {
}
*/

// Implement viewDidLoad to do additional setup after loading the view, typically from a nib.
- (void)viewDidLoad {
    [super viewDidLoad];
	NSUserDefaults *user = [NSUserDefaults standardUserDefaults];
	
	// Get the server key from the Settings Application, if first time
	server = [user stringForKey:@"server_address"];
	if(!server) {
		// If the default value doesn't exist then it hasn't been set. We need to loop
		// through the default values in the Root.plist and set them programtically.
		[self registerDefaultsFromSettingsBundle];
		server = [user stringForKey:@"server_address"];
	}	
	NSLog(@"Server address: %@", server);
	serverTextView.text = server;
	
	robot = [[Robot alloc] initWithServer:server]; 
	[robot retain]; // detach will release objects early

	// Start a NSTimer and hand it the update: method
	timer = [[NSTimer scheduledTimerWithTimeInterval:(1.0 / 60.0)		// Update 1/60th second
											  target:self
											selector:@selector(update:)
											userInfo:nil
											 repeats:YES] retain];
	
	// Start a new thread and run the robot code.
	[NSThread detachNewThreadSelector: @selector(robotRun:) toTarget: robot withObject: self];	
}

// Rotate into landscape mode by default, override default behaviour 
- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)toInterfaceOrientation {
    // Return YES for supported orientations
    return (toInterfaceOrientation == UIInterfaceOrientationLandscapeRight);
}


- (void)didReceiveMemoryWarning {
	// Releases the view if it doesn't have a superview.
    [super didReceiveMemoryWarning];
	
	// Release any cached data, images, etc that aren't in use.
}

- (void)viewDidUnload {
	// Release any retained subviews of the main view.
	// e.g. self.myOutlet = nil;
}


- (void)dealloc {
    [super dealloc];
	[server release];
	[robot release];
	[timer invalidate];		// stop timer updates
	[timer release];
}

- (void)registerDefaultsFromSettingsBundle {
    NSString *settingsBundle = [[NSBundle mainBundle] pathForResource:@"Settings" ofType:@"bundle"];
    if(!settingsBundle) {
        NSLog(@"Could not find Settings.bundle");
        return;
    }
	
    NSDictionary *settings = [NSDictionary dictionaryWithContentsOfFile:[settingsBundle stringByAppendingPathComponent:@"Root.plist"]];
    NSArray *preferences = [settings objectForKey:@"PreferenceSpecifiers"];
	
    NSMutableDictionary *defaultsToRegister = [[NSMutableDictionary alloc] initWithCapacity:[preferences count]];
    for(NSDictionary *prefSpecification in preferences) {
        NSString *key = [prefSpecification objectForKey:@"Key"];
        if(key) {
            [defaultsToRegister setObject:[prefSpecification objectForKey:@"DefaultValue"] forKey:key];
        }
    }
	
    [[NSUserDefaults standardUserDefaults] registerDefaults:defaultsToRegister];
    [defaultsToRegister release];
}


@end
