//
//  Robot.mm
//	RobotSample
//
//	Feel free to modify the class and let me know what you do.
//
//  Created by Paul Solt, paulsolt@gmail.com on 4/30/09.
//  Copyright 2009 RIT. All rights reserved.
//

#import "Robot.h"

@implementation Robot

@synthesize speed;
@synthesize turnRate;
@synthesize robotOn;
@synthesize server;

// Initialize the Robot object with a given server address.
- (id)initWithServer:(NSString *)serverAddress {
	
	if(self = [super init]) {
		speed = 0.0;
		turnRate = 0.0;
		robotOn = YES;
		server = serverAddress;
	}
	return self;
}

- (void) robotRun:(id)sender {
	
	NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
	
	const char *cServer = [server UTF8String];	// Get a c string
	
	try {
		
		PlayerClient robot(cServer);
		Position2dProxy pp(&robot, 0);
		
		pp.SetMotorEnable(true);			// Turn the motor on
		
		while(robotOn) {
			robot.Read();	// Read robot data
			
			// Pass messages back to main thread
			[sender performSelectorOnMainThread:@selector(displayText:)
								withObject:[NSString stringWithFormat:@"Speed: %5.3f\n Angle: %5.3f\nPosition: %5.3f %5.3f",
											speed, turnRate,pp.GetXPos(),pp.GetYPos()]
								waitUntilDone:NO];
			
			// Randomly update the rotation and speed of the robot.
			turnRate = drand48();
			speed = drand48()/2.0;
			
			pp.SetSpeed(speed, turnRate);
			
		}
		[sender performSelectorOnMainThread:@selector(connectionClosedWithMessage:)
								 withObject:@"Robot Disconnected Restart"
							  waitUntilDone:NO];
	} catch(PlayerError p) {
		std::cout << "ROBOT Error: " <<  p << std::endl;
		
		NSString *errorMsg = [NSString stringWithCString:p.GetErrorStr().c_str()];
		[sender performSelectorOnMainThread:@selector(connectionClosedWithMessage:)
								 withObject:errorMsg
							  waitUntilDone:NO];
	}
	
	NSLog(@"Disconnected");
	
	[pool release];	
}

- (void)dealloc {
	[server release];
	[super dealloc];
}



@end
